
declare var PUBLIC_URL: string;
console.log("Url is " + PUBLIC_URL);


import { TimeView } from "./timeview";
import { PlantView, PlantViews } from "./plant";


export class Controller{
  private readonly timeView: TimeView
  readonly plantViews:PlantViews
  constructor(){
    this.timeView = new TimeView()
    this.plantViews = new PlantViews(this)
  }

  configChanged() {
    updateJson()
  }

  testPlant(plantId: number) {
    var body: TestPump = {
      pump: plantId
    }
    var pretty = JSON.stringify(body, undefined, 1);
    fetch(PUBLIC_URL + "/pumptest", {
      method: "POST",
      body: pretty
    })
      .then(response => response.text())
      .then(text => console.log(text))
  }

  updateRealTimeData() {
    fetch(PUBLIC_URL + "/data")
      .then(response => response.json())
      .then(json => json as GetData)
      .then(time => {
        controller.timeView.update(time.native, time.rtc)
  
        time.moisture_a.forEach((a, index) => {
          controller.plantViews.getPlant(index).setMoistureA(a);
        })
        time.moisture_b.forEach((b, index) => {
          controller.plantViews.getPlant(index).setMoistureB(b);
        })
  
        setTimeout(controller.updateRealTimeData, 1000);
      })
      .catch(error => {
        console.log(error);
        setTimeout(controller.updateRealTimeData, 10000);
      });
  }

}

const controller = new Controller();
setTimeout(controller.updateRealTimeData, 1000);





let scanWifiBtn = document.getElementById("scan") as HTMLButtonElement;
if (scanWifiBtn) {
  scanWifiBtn.onclick = scanWifi;
}


export function scanWifi() {
  var scanButton = (document.getElementById("scan") as HTMLButtonElement);
  scanButton.disabled = true;

  var ajax = new XMLHttpRequest();
  ajax.responseType = 'json';
  ajax.onreadystatechange = () => {
    if (ajax.readyState === 4) {
      callback(ajax.response);
    }
  };
  ajax.onerror = (evt) => {
    console.log(evt)
    scanButton.disabled = false;
    alert("Failed to start see console")
  }
  ajax.open("POST", PUBLIC_URL + "/wifiscan");
  ajax.send();
}

function callback(data: SSIDList) {
  var ssidlist = document.getElementById("ssidlist") as HTMLElement
  ssidlist.innerHTML = ''

  for (var ssid of data.ssids) {
    var wi = document.createElement("option");
    wi.value = ssid;
    ssidlist.appendChild(wi);
  }
}

function addTimeOptions(select: HTMLSelectElement) {
  for (let i = 0; i < 24; i++) {
    let option = document.createElement("option");
    option.innerText = i.toString();
    select.appendChild(option);
  }
}

var ap_ssid = (document.getElementById("ap_ssid") as HTMLInputElement);
ap_ssid.onchange = updateJson
var ssid = (document.getElementById("ssid") as HTMLInputElement);
ssid.onchange = updateJson
var password = (document.getElementById("password") as HTMLInputElement);
password.onchange = updateJson
let mqtt_url = document.getElementById("mqtt_url") as HTMLInputElement;
mqtt_url.onchange = updateJson
let base_topic = document.getElementById("base_topic") as HTMLInputElement;
base_topic.onchange = updateJson
let max_consecutive_pump_count = document.getElementById("max_consecutive_pump_count") as HTMLInputElement;
max_consecutive_pump_count.onchange = updateJson

let tank_useable_ml = document.getElementById("tank_useable_ml") as HTMLInputElement;
tank_useable_ml.onchange = updateJson
let tank_empty_percent = document.getElementById("tank_empty_percent") as HTMLInputElement;
tank_empty_percent.onchange = updateJson
let tank_full_percent = document.getElementById("tank_full_percent") as HTMLInputElement;
tank_full_percent.onchange = updateJson
let tank_warn_percent = document.getElementById("tank_warn_percent") as HTMLInputElement;
tank_warn_percent.onchange = updateJson
let tank_sensor_enabled = document.getElementById("tank_sensor_enabled") as HTMLInputElement;
tank_sensor_enabled.onchange = updateJson
let tank_allow_pumping_if_sensor_error = document.getElementById("tank_allow_pumping_if_sensor_error") as HTMLInputElement;
tank_allow_pumping_if_sensor_error.onchange = updateJson
let night_lamp_only_when_dark = document.getElementById("night_lamp_only_when_dark") as HTMLInputElement;
night_lamp_only_when_dark.onchange = updateJson
let night_lamp_time_start = document.getElementById("night_lamp_time_start") as HTMLSelectElement;
night_lamp_time_start.onchange = updateJson
addTimeOptions(night_lamp_time_start);
let night_lamp_time_end = document.getElementById("night_lamp_time_end") as HTMLSelectElement;
night_lamp_time_end.onchange = updateJson
addTimeOptions(night_lamp_time_end);


function updateJson() {
  var current: PlantControllerConfig = {
    ap_ssid: ap_ssid.value,
    ssid: ssid.value,
    password: password.value,
    max_consecutive_pump_count: +max_consecutive_pump_count.value,
    mqtt_url: mqtt_url.value,
    base_topic: base_topic.value,
    tank_allow_pumping_if_sensor_error: tank_allow_pumping_if_sensor_error.checked,
    tank_sensor_enabled: tank_sensor_enabled.checked,
    tank_useable_ml: +tank_useable_ml.value,
    tank_warn_percent: +tank_warn_percent.value,
    tank_empty_percent: +tank_empty_percent.value,
    tank_full_percent: +tank_full_percent.value,
    night_lamp_hour_start: +night_lamp_time_start.value,
    night_lamp_hour_end: +night_lamp_time_end.value,
    night_lamp_only_when_dark: night_lamp_only_when_dark.checked,
    plants: []
  }

  for (let i = 0; i < 8; i++) {
    current.plants[i] = controller.plantViews.getPlant(i).getConfig();
  }
  //sync(current);
  console.log(current);

  var pretty = JSON.stringify(current, undefined, 1);
  console.log(pretty)
  //json.value = pretty;
}

let fromWrapper = (() => {

  let plantcount = 0;




  let json = document.getElementById('json') as HTMLInputElement

  
  function sync(current: PlantControllerConfig) {
    plantcount = current.plants.length

    ap_ssid.value = current.ap_ssid;
    ssid.value = current.ssid;
    password.value = current.password;

    mqtt_url.value = current.mqtt_url;
    base_topic.value = current.base_topic;
    max_consecutive_pump_count.value = current.max_consecutive_pump_count.toString();

    tank_useable_ml.disabled = !current.tank_sensor_enabled;
    tank_warn_percent.disabled = !current.tank_sensor_enabled;
    tank_sensor_enabled.checked = current.tank_sensor_enabled;
    tank_allow_pumping_if_sensor_error.checked = current.tank_allow_pumping_if_sensor_error;
    tank_useable_ml.value = current.tank_useable_ml.toString();
    tank_warn_percent.value = current.tank_warn_percent.toString();
    tank_empty_percent.value = current.tank_empty_percent.toString();
    tank_full_percent.value = current.tank_full_percent.toString();

    night_lamp_time_start.value = current.night_lamp_hour_start.toString();
    night_lamp_time_end.value = current.night_lamp_hour_end.toString();

    for (let i = 0; i < current.plants.length; i++) {
      const plantConfig = current.plants[i];
      const plantView = controller.plantViews.getPlant(i);
      plantView.setConfig(plantConfig)
    }
  }


  let submitFormBtn = document.getElementById("submit") as HTMLButtonElement
  let submit_status = document.getElementById("submit_status") as HTMLElement

  if (submitFormBtn) {

    submitFormBtn.onclick = function () {
      updateJson()
      fetch(PUBLIC_URL + "/set_config", {
        method: "POST",
        body: json.value,

      })
        .then(response => response.text())
        .then(text => submit_status.innerText = text)

    };

  }

  fetch(PUBLIC_URL + "/get_config")
    .then(response => response.json())
    .then(loaded => {
      var currentConfig = loaded as PlantControllerConfig;
      sync(currentConfig);
      var pretty = JSON.stringify(currentConfig, undefined, 1);
      json.value = pretty;
    }
    )
})
fromWrapper()



export function uploadFile() {
  var file1 = document.getElementById("file1") as HTMLInputElement;
  var loaded_n_total = document.getElementById("loaded_n_total") as HTMLElement;
  var progressBar = document.getElementById("progressBar") as HTMLProgressElement;
  var status = document.getElementById("status") as HTMLElement;
  var answer = document.getElementById("answer") as HTMLElement;

  if (file1.files == null){
    //TODO error dialog here
    return 
  }

  var file = file1.files[0];
  var ajax = new XMLHttpRequest();


  ajax.upload.addEventListener("progress", event => {
    loaded_n_total.innerHTML = "Uploaded " + event.loaded + " bytes of " + event.total;
    var percent = (event.loaded / event.total) * 100;
    progressBar.value = Math.round(percent);
    status.innerHTML = Math.round(percent) + "%";
    answer.innerHTML = "in progress";
  }, false);
  ajax.addEventListener("load", () => {
    status.innerHTML = ajax.responseText;
    answer.innerHTML = "finished";
    progressBar.value = 0;
  }, false);
  ajax.addEventListener("error", () => {
    status.innerHTML = ajax.responseText;
    answer.innerHTML = "failed";
  }, false);
  ajax.addEventListener("abort", () => {
    status.innerHTML = ajax.responseText;
    answer.innerHTML = "aborted";
  }, false);
  ajax.open("POST", PUBLIC_URL + "/ota");
  ajax.send(file);
}

let file1Upload = document.getElementById("file1") as HTMLInputElement;
file1Upload.onchange = uploadFile;

let firmware_buildtime = document.getElementById("firmware_buildtime") as HTMLDivElement;
let firmware_githash = document.getElementById("firmware_githash") as HTMLDivElement;

document.addEventListener('DOMContentLoaded', function () {
  fetch(PUBLIC_URL + "/version")
    .then(response => response.json())
    .then(json => json as VersionInfo)
    .then(versionInfo => {
      firmware_buildtime.innerText = versionInfo.build_time;
      firmware_githash.innerText = versionInfo.git_hash;
    })
}, false);


let battery_flash_button = document.getElementById("battery_flash_button") as HTMLButtonElement;
let battery_flash_file = document.getElementById("battery_flash_file") as HTMLInputElement;
let battery_flash_message = document.getElementById("battery_flash_message") as HTMLElement;
let battery_flash_progressBar = document.getElementById("battery_flash_progressBar") as HTMLProgressElement;
let battery_flash_loaded_n_total = document.getElementById("battery_flash_loaded_n_total") as HTMLElement;
let battery_flash_status = document.getElementById("battery_flash_status") as HTMLElement;


var ajax = new XMLHttpRequest();

ajax.upload.addEventListener("progress", event => {
  battery_flash_loaded_n_total.innerHTML = "Uploaded " + event.loaded + " bytes of " + event.total;
  var percent = (event.loaded / event.total) * 100;
  battery_flash_progressBar.value = Math.round(percent);
  battery_flash_status.innerHTML = Math.round(percent) + "%";
  battery_flash_message.innerHTML = "in progress";
}, false);
ajax.addEventListener("load", () => {
  battery_flash_status.innerHTML = ajax.responseText;
  battery_flash_message.innerHTML = "finished";
  battery_flash_progressBar.value = 0;
}, false);
ajax.addEventListener("error", () => {
  battery_flash_status.innerHTML = ajax.responseText;
  battery_flash_message.innerHTML = "failed";
}, false);
ajax.addEventListener("abort", () => {
  battery_flash_status.innerHTML = ajax.responseText;
  battery_flash_message.innerHTML = "aborted";
}, false);




battery_flash_button.onclick = async function () {
  //ajax.open("POST", "/flashbattery");
  //ajax.send(battery_flash_file.files[0]);

  ajax.open("POST", PUBLIC_URL + "/flashbattery?flash=true");
  ajax.send(battery_flash_file.files![0]);
};
